﻿using System;
using System.IO;

namespace ImpersonateCLI
{
    public class Program
    {
        public static void Main(string[] args)
        {
            if (args.Length < 3)
            {
                PrintHelpText();
                Console.WriteLine("WARNING! Not enough arguments to continue.");
                Environment.Exit(1);
                return;
            }
            string userName = args[0].Replace("/", "\\");
            string domain = null;
            string password = args[1];
            if (userName.Contains("\\"))
            {
                domain = userName.Split('\\')[0];
                userName = userName.Split('\\')[1];
            }
            if (domain != null)
            {
                Console.WriteLine("Using domain: " + domain);
            }
            Console.WriteLine("Using user name: " + userName);
            var passwordRepresentation = "";
            for (var i = 0; i < password.Length; i++)
            {
                passwordRepresentation += "*";
            }
            Console.WriteLine("Using password: " + passwordRepresentation);
            UserCredentials credentials = null;
            if (domain == null) {
                credentials = new UserCredentials(userName, password);
            } else
            {
                credentials = new UserCredentials(domain, userName, password);
            }
            var program = args[3];
            var cliArguments = new string[args.Length - 3];
            if (args.Length > 3)
            {
                var enumerator = 0;
                for(var y = 3; y < args.Length; y++)
                {
                    cliArguments[enumerator] = (args[y]);
                }
            }
            if (cliArguments.Length == 0) cliArguments = null;
            try
            {
                Impersonation.RunAsUser(credentials, LogonType.Interactive, () => System.Diagnostics.Process.Start(program, string.Join(" ", cliArguments)));
            }catch(Exception e)
            {
                Console.WriteLine();
                Console.WriteLine("EXCEPTION ENCOUNTERED:" + Environment.NewLine + e.Message);
            }
        }

        private static void PrintHelpText()
        {
            Console.WriteLine();
            Console.WriteLine("Impersonate anything");
            Console.WriteLine("==========================");
            Console.WriteLine("Utility to switch Windows user for a specific CLI command. ");
            Console.WriteLine();
            Console.WriteLine("Usage:");
            var path = Path.GetFileName(System.Diagnostics.Process.GetCurrentProcess().MainModule.FileName);
            Console.WriteLine("   " + path + " [domainName\\]userName password command [commandArgument1]");
            Console.WriteLine();
            Console.WriteLine("Any trailing arguments will be treated as command parameters.");
            Console.WriteLine();
        }
    }
}
